@@ -1,5 +1,8 @@
 #!/usr/bin/env python3
-import pathlib, pandas, math, sys,  re, logging
+import pathlib, pandas, math, sys,  re, logging, numpy
+import warnings
+pandas.options.mode.chained_assignment = None
+# from pandas.core.algorithms import isin
 from abritamr.CustomLog import CustomFormatter
 
 class Collate:
@@ -8,7 +11,7 @@ class Collate:
     a base class for collation of amrfinder results - to be used when not doing MDU QC
     """
     
-
+    ANNOTATIONS = {'blast':'*','partial':'^','exact':''}
     REFGENES = pathlib.Path(__file__).parent / "db" / "refgenes_latest.csv"
     MATCH = ["ALLELEX", "BLASTX", "EXACTX", "POINTX"]
     NONRTM = [
@@ -64,7 +67,7 @@ class Collate:
         """
         for i in dict_for_joining:
             if i != "Isolate":
-                dict_for_joining[i] = list(set(dict_for_joining[i]))
+                dict_for_joining[i] = sorted(list(set(dict_for_joining[i])))
                 dict_for_joining[i] = ",".join(dict_for_joining[i])
 
         return dict_for_joining
@@ -76,16 +79,29 @@ class Collate:
         """
         gene_id_col = "Gene symbol" if colname != "refseq_protein_accession" else "Accession of closest sequence"
         
-        d = reftab[reftab[colname] == row[1][gene_id_col]]['enhanced_subclass'].values[0]
-        
+        try:
+            d = reftab[reftab[colname] == row[1][gene_id_col]]['enhanced_subclass'].values[0]
+        except:
+            d = '-'
         if d in self.NONRTM:
             return self.RTM
         elif d in self.MACROLIDES:
             return self.MAC
         elif d == "-":
-            return reftab[reftab[colname] == row[1]["Gene symbol"]][
-                "subclass"
-            ].unique()[0]
+            try:
+                return reftab[reftab[colname] == row[1]["Gene symbol"]][
+                    "subclass"
+                ].unique()[0]
+            except:
+                # genbank_protein_accession
+                self.logger.warning(f"There appears to be no refseq accession for this so we will try genbank.")
+                to_return = 'Unknown'
+                for i in ['genbank_protein_accession','refseq_nucleotide_accession','genbank_nucleotide_accession']:
+                    if len(reftab[reftab[i] == row[1]["Gene symbol"]]["subclass"].unique()) >= 1:
+                        return reftab[reftab[i] == row[1]["Gene symbol"]]["subclass"].unique()[0]
+                return to_return
+            
+                
         else:
             return d
 
@@ -96,13 +112,18 @@ class Collate:
         return reftab[reftab["refseq_protein_accession"] == protein]["gene_family"].values[0]
 
     def extract_gene_name(self, protein, reftab):
-
-        if reftab[reftab["refseq_protein_accession"] == protein]["allele"].values[0] != '-':
-            return reftab[reftab["refseq_protein_accession"] == protein]["allele"].values[0]
-        else:
-            return reftab[reftab["refseq_protein_accession"] == protein]["gene_family"].values[0]
+        
+        try:
+            if reftab[reftab["refseq_protein_accession"] == protein]["allele"].values[0] != '-':
+                return reftab[reftab["refseq_protein_accession"] == protein]["allele"].values[0]
+            else:
+                return reftab[reftab["refseq_protein_accession"] == protein]["gene_family"].values[0]
+        except IndexError:
+            for i in ['genbank_protein_accession','refseq_nucleotide_accession','genbank_nucleotide_accession']:
+                if len(reftab[reftab[i] == protein]["gene_family"].unique()) >= 1:
+                    return reftab[reftab[i] == protein]["gene_family"].unique()[0]
             
-    def setup_dict(self, drugclass_dict, reftab, row):
+    def setup_dict(self, drugclass_dict, reftab, row, _type = 'exact'):
         """
         return the dictionary for collation
         """
@@ -135,15 +156,15 @@ class Collate:
 
         return drugclass_dict
 
-    def _virulence_dict(self, virulence_dict, row):
+    def _other_dict(self, other_dict, row):
         """
-        report virulence factors
+        report virulence and stress genes
         """
-        if row[1]['Element subtype'] in virulence_dict:
-            virulence_dict[row[1]['Element subtype']].append(row[1]['Gene symbol'])
+        if row[1]['Element subtype'].capitalize() in other_dict:
+            other_dict[row[1]['Element subtype'].capitalize()].append(row[1]['Gene symbol'])
         else:
-            virulence_dict[row[1]['Element subtype']] = [row[1]['Gene symbol']]
-        return virulence_dict
+            other_dict[row[1]['Element subtype'].capitalize()] = [row[1]['Gene symbol']]
+        return other_dict
 
     def get_per_isolate(self, reftab, df, isolate):
         """
@@ -151,31 +172,91 @@ class Collate:
         """
         drugclass_dict = {"Isolate": isolate}
         partials = {"Isolate": isolate}
-        virulence = {"Isolate": isolate}
+        other = {"Isolate": isolate}
         for row in df.iterrows():
             # print(row)
             # if the match is good then generate a drugclass dict
             if row[1]["Gene symbol"] == "aac(6')-Ib-cr" and row[1]["Method"] in ["EXACTX", "ALLELEX"]: # This is always a partial - unclear
                 partials = self.setup_dict(drugclass_dict = partials, reftab = reftab, row = row)
-            elif row[1]["Method"] in self.MATCH and row[1]["Scope"] == "core" and row[1]["Element type"] == "AMR":
+            elif row[1]["Method"] in self.MATCH and row[1]["Element type"] == "AMR" and row[1]['Element subtype'] != "AMR-SUSCEPTIBLE":
                 drugclass_dict = self.setup_dict(drugclass_dict = drugclass_dict, reftab = reftab, row = row)
-            elif row[1]["Method"] not in self.MATCH and row[1]["Scope"] == "core" and row[1]["Element type"] == "AMR":
+            elif row[1]["Method"] not in self.MATCH and row[1]["Element type"] == "AMR" and row[1]['Element subtype'] != "AMR-SUSCEPTIBLE":
                 partials = self.setup_dict(drugclass_dict = partials, reftab = reftab, row = row)
-            elif row[1]["Method"] in self.MATCH and row[1]['Element type'] == 'VIRULENCE':
-                virulence = self._virulence_dict(virulence_dict = virulence, row = row)
+            else:
+                other = self._other_dict(other_dict = other, row = row)
+            
         drugclass_dict = self.joins(dict_for_joining=drugclass_dict)
         partials = self.joins(dict_for_joining=partials)
-        virulence = self.joins(dict_for_joining = virulence)
-        return drugclass_dict, partials, virulence
+        other = self.joins(dict_for_joining = other)
+        return drugclass_dict, partials, other
+
+    def _get_cols(self, df, _not = 'Isolate'):
+        
+        cols = [c for c in df.columns if c != _not]
+
+        return cols
+
+    def _add_caret(self,df,cols):
+
+        for c in cols:
+            df[c] = numpy.where(df[c]!="", df[c].str.strip("*") + "^", df[c])
+        return df
+
+    def _merge(self, match, partial, pcols, mcols):
+
+        df = pandas.DataFrame()
+        tmp = match.merge(partial, on = 'Isolate', how = 'outer')
+        tmp_cols = self._get_cols(df = tmp)
+        for t in tmp_cols:
+            if t.endswith('_x') or t.endswith('_y'):
+                if df.empty:
+                    df = tmp[['Isolate']]
+                c = t.split('_')[0]
+                if c not in list(df.columns):
+                    df[c] = tmp[[f"{c}_x", f"{c}_y"]].apply(lambda x: ','.join(x), axis = 1)
+            else:
+                if df.empty:
+                    df = tmp[['Isolate', t]]
+                else:
+                    df = df.merge(tmp[['Isolate', t]], on = 'Isolate',how = 'outer') 
+        
+        return df
+
+    def _combine_dfs(self, match, partial, virulence):
+        
+        match = match.fillna('')
+        partial = partial.fillna('')
+        pcols = self._get_cols(df = partial)
+        mcols = self._get_cols(df = match)
+        vcols = self._get_cols(df = virulence)
+
+        if vcols == [] and pcols == [] and mcols == []:
+            return pandas.DataFrame()
+
+        if pcols != []:
+            partial = self._add_caret(df = partial, cols = pcols)      
+        
+        df = self._merge(match = match, partial = partial, mcols = mcols, pcols = pcols)
+
+        if isinstance(virulence, pandas.DataFrame):
+            df = df.merge(virulence, on = 'Isolate', how = 'outer')      
+
+        return df
 
-    
     def save_files(self, path, match, partial, virulence):
         
         files = {'summary_matches.txt': match, 'summary_partials.txt': partial, 'summary_virulence.txt':virulence}
+        
         for f in files:
             out = f"{path}/{f}" if path != '' else f"{f}"
             self.logger.info(f"Saving {out}")
             files[f].set_index('Isolate').to_csv(f"{out}", sep = '\t')
+        combd = self._combine_dfs(match = match, partial = partial, virulence = virulence)
+        combd_out = f"{path}/abritamr.txt" if path != '' else f"abritamr.txt"
+        self.logger.info(f"Saving combined file : {combd_out}")
+
+        combd.set_index('Isolate').to_csv(f"{combd_out}", sep = '\t')
+        
         return True
         
 
@@ -358,7 +439,13 @@ class MduCollate(Collate):
         if col in [ 'Beta-lactamase (not ESBL or carbapenemase)', 'Beta-lactamase (narrow-spectrum)']:
             return gene
         return ''
-    
+
+    def _chloramphenicol_res_sal(self, col, gene):
+
+        if 'phenicol' in col:
+            return gene
+        return ''
+
     def _cefo_esbl_res_sal(self, col, gene):
 
         if col == 'ESBL':
@@ -373,7 +460,7 @@ class MduCollate(Collate):
     
     def _carbapenem_res_salmo(self, col, gene):
 
-        if col in ["Carbapenemase", "Carbapenemase (MBL)", "Carbapenemase OXA-51 family"]:
+        if "Carbapenemase" in col:
             return gene
         return ''
 
@@ -456,6 +543,7 @@ class MduCollate(Collate):
             "Ciprofloxacin":self._cipro_res_salmo,
             "Sulfathiazole":self._sulf_res_salmo,
             "Trimethoprim":self._trimet_res_salmo,
+            "Chloramphenicol":self._chloramphenicol_res_sal,
             "Aminoglycosides (Ribosomal methyltransferase)": self._rmt_res_salmo,
             "Colistin":self._colistin_res_salmo
         }
@@ -472,6 +560,7 @@ class MduCollate(Collate):
             "Ciprofloxacin":[],
             "Azithromycin":[],
             "Carbapenem":[],
+            "Chloramphenicol":[],
             "Aminoglycosides (Ribosomal methyltransferase)":[],"Colistin":[], "Other":[]
         }
         # group drugs
@@ -541,8 +630,8 @@ class MduCollate(Collate):
         ]
 
         
-        van_match = re.compile("van[A,B,C,D,E,G,L,M,N][\S]*")
-        mec_match = re.compile("mec[^IR]")
+        van_match = re.compile(r"van[A,B,C,D,E,G,L,M,N][\S]*")
+        mec_match = re.compile(r"mec[^IR]")
         
         genes_reported = []  # genes for reporting
         genes_not_reported = []  # genes found but not reportable
