@@ -26,24 +26,27 @@ class RunFinder(object):
         self.run_type = args.run_type
         self.jobs = args.jobs
         self.prefix = args.prefix
+        self.identity = args.identity
         self.amrfinder_db = os.environ.get('AMRFINDER_DB')
 
     def _batch_cmd(self):
         """
         generate cmd with parallel
         """
-        org = f"--plus --organism {self.organism}" if self.organism != '' else ''
+        org = f"--organism {self.organism}" if self.organism != '' else ''
         d = f" -d {self.amrfinder_db}" if self.amrfinder_db != '' else ''
-        cmd = f"parallel -j {self.jobs} --colsep '\\t' 'mkdir -p {{1}} && amrfinder -n {{2}} -o {{1}}/amrfinder.out {org} --threads 1{d}' :::: {self.input}"
+        _id = f" --ident_min {self.identity} " if self.identity != '' else ''
+        cmd = f"parallel -j {self.jobs} --colsep '\\t' 'mkdir -p {{1}} && amrfinder -n {{2}} -o {{1}}/amrfinder.out --plus {org} --threads 1{d}{_id}' :::: {self.input}"
         return cmd
     
     def _single_cmd(self):
         """
         generate a single amrfinder command
         """
-        org = f"--plus --organism {self.organism}" if self.organism != '' else ''
+        org = f"--organism {self.organism}" if self.organism != '' else ''
         d = f" -d {self.amrfinder_db}" if self.amrfinder_db else ''
-        cmd = f"mkdir -p {self.prefix} && amrfinder -n {self.input} -o {self.prefix}/amrfinder.out {org} --threads {self.jobs}{d}"
+        _id = f" --ident_min {self.identity} " if self.identity != '' else ''
+        cmd = f"mkdir -p {self.prefix} && amrfinder -n {self.input} -o {self.prefix}/amrfinder.out --plus {org} --threads {self.jobs}{d}{_id}"
         return cmd
     
     def _check_amrfinder(self):
@@ -51,12 +54,14 @@ class RunFinder(object):
         Check that amrfinder is installed and db setup properly.
         """
         ok = False
+        self.logger.info(f"Checking for amrfinder DB: {self.amrfinder_db} and comparing it to {self.db}")
         if self.amrfinder_db == '' or self.amrfinder_db == None:
             self.logger.warning(f"It seems you don't have the AMRFINDER_DB variable set. Now checking AMRfinder setup. Please note if the AMRFinder DB is not v {self.db} this may cause errors")
-            cmd = f"amrfinder --debug"
+            cmd = f"amrfinder --help"
             pat = re.compile(r'(?P<id>[0-9]{4}-[0-9]{5,6})-?(?P<itemcode>.{1,2})?')
-            p = subprocess.run("amrfinder --debug", shell = True, encoding = "utf-8", capture_output = True)
+            p = subprocess.run(cmd, shell = True, encoding = "utf-8", capture_output = True)
             m = re.search(r'[0-9]{4}-[0-9]{2}-[0-9]{2}', p.stderr)
+            
             if m:
                 ok = True
             else:
@@ -117,14 +122,15 @@ class RunFinder(object):
         """
         if self._check_amrfinder():
             self.logger.info(f"All check complete now running AMRFinder")
-            cmd = self._generate_cmd()
-            self.logger.info(f"You are running abritamr in {self.run_type} mode. Now executing : {cmd}")
-            self._run_cmd(cmd)
-            self._check_outputs()
+            
             
         else:
-            self.logger.critical(f"It seems that amrfinder is not properly configured. Please check amrfinder documentation (https://github.com/ncbi/amr/wiki/AMRFinderPlus-database) and try again.")
-            raise SystemExit
+            self.logger.critical(f"Your amrfinder database version is NOT {self.db}. abriTAMR will still run but behaviour may not be as expected in terms of binnig genes into the appropriate drug classes.")
+            # raise SystemExit
+        cmd = self._generate_cmd()
+        self.logger.info(f"You are running abritamr in {self.run_type} mode. Now executing : {cmd}")
+        self._run_cmd(cmd)
+        self._check_outputs()
         Data = collections.namedtuple('Data', ['run_type', 'input', 'prefix'])
         amr_data = Data(self.run_type, self.input, self.prefix)
 
